#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <class_info.h>
#include <jmp.h>

#define BLEN 32
static char buf[BLEN];

static char* get_class_id (cls* c) {
    snprintf (buf, BLEN, "%p", cls_get_class_id (c));
    return buf;    
}

static char* get_super_class_name (cls* c) {
    cls* s = cls_get_super (c);
    if (s == NULL) 
	s = get_super_class (c);
    if (s != NULL)
	return cls_get_name (s);
    return _("<unknown>");
}

static char* get_n_interfaces (cls* c) {
    snprintf (buf, BLEN, "%d", c->n_interfaces);
    return buf;
}

static char* get_n_statics (cls* c) {
    snprintf (buf, BLEN, "%d", c->n_statics);
    return buf;
}

static char* get_n_instances (cls* c) {
    snprintf (buf, BLEN, "%d", c->n_instances);
    return buf;
}

static void add_fields_to_frame (int num, field* fields, GtkWidget* frametable) {
    GtkWidget* label;
    int j = 0;
    for (j = 0; j < num; j++) {
	label = gtk_label_new (get_field_type (&fields[j]));
	gtk_table_attach (GTK_TABLE (frametable), label, 0, 1, j, j + 1, 
			  GTK_FILL, 0, 0, 0);
	gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	
	label = gtk_label_new (fields[j].field_name);
	gtk_table_attach (GTK_TABLE (frametable), label, 1, 2, j, j + 1, 
			  GTK_FILL, 0, 0, 0);
	gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    }
}

static void add_interfaces_to_frame (int num, cls** interfaces, GtkWidget* frametable) {
    GtkWidget* label;
    int j = 0;
    for (j = 0; j < num; j++) {
	label = gtk_label_new (cls_get_name (interfaces[j]));
	gtk_table_attach (GTK_TABLE (frametable), label, 0, 1, j, j + 1, 
			  GTK_FILL, 0, 0, 0);
	gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    }    
}

/** method used to show a window with class info.. */
void show_class_info (cls* c) {
    GtkWidget* win;
    GtkWidget* table;
    GtkWidget* frame;
    GtkWidget* frametable;
    GtkWidget* label;
    int i = 0;

    gchar *labels[] = { _("Class"),
			_("Class id"),
			_("Super class"), 
			_("Source file"), 
			_("#Interfaces"),
			_("#Static vars"),
			_("#Instance vars"),
			NULL};
    
    char* ((*mp[])(cls* m)) = { cls_get_name,
				get_class_id,
				get_super_class_name, 
				cls_get_source_name,
				get_n_interfaces,
				get_n_statics,
				get_n_instances,
				NULL};
    

    win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_object_set_data (GTK_OBJECT (win), "win", win);
    gtk_window_set_title (GTK_WINDOW (win), _("Class info"));

    table = gtk_table_new (7, 2, FALSE);
    gtk_table_set_row_spacings (GTK_TABLE (table), 2);
    gtk_table_set_col_spacings (GTK_TABLE (table), 2);
    gtk_container_add (GTK_CONTAINER (win), table);
    
    while (labels[i] != NULL) {
	label = gtk_label_new (labels[i]);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i + 1, 
			  GTK_FILL, 0, 0, 0);
	gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    
	label = gtk_label_new (mp[i](c));
	gtk_table_attach (GTK_TABLE (table), label, 1, 2, i, i + 1, 
			  GTK_FILL, 0, 0, 0);
	gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	i++;
    } 
    
    frame = gtk_frame_new (_("Interfaces"));
    frametable = gtk_table_new (c->n_interfaces, 1, FALSE);
    gtk_container_add (GTK_CONTAINER (frame), frametable);
    gtk_table_set_row_spacings (GTK_TABLE (frametable), 2);
    gtk_table_attach (GTK_TABLE (table), frame,  0, 2, i, i + 1, 
		      GTK_FILL, 0, 0, 0);
    add_interfaces_to_frame (c->n_interfaces, c->interfaces, frametable);
    
    i++;
    frame = gtk_frame_new (_("Static vars"));
    frametable = gtk_table_new (c->n_statics, 2, FALSE);
    gtk_container_add (GTK_CONTAINER (frame), frametable);
    gtk_table_set_row_spacings (GTK_TABLE (frametable), 2);
    gtk_table_set_col_spacings (GTK_TABLE (frametable), 10);
    gtk_table_attach (GTK_TABLE (table), frame,  0, 2, i, i + 1, 
		      GTK_FILL, 0, 0, 0);
    add_fields_to_frame (c->n_statics, c->statics, frametable);
    
    i++;
    frame = gtk_frame_new (_("Instance vars"));
    frametable = gtk_table_new (c->n_instances, 2, FALSE);
    gtk_container_add (GTK_CONTAINER (frame), frametable);
    gtk_table_set_row_spacings (GTK_TABLE (frametable), 2);
    gtk_table_set_col_spacings (GTK_TABLE (frametable), 10);
    gtk_table_attach (GTK_TABLE (table), frame,  0, 2, i, i + 1, 
		      GTK_FILL, 0, 0, 0);    
    add_fields_to_frame (c->n_instances, c->instances, frametable);
    gtk_widget_show_all (win);
}
